{*******************************************************
                TShadowPanel [shdowpnl unit]

This unit contains the code for the TShadowPanel component.
TShadowPanel is a TCustomPanel decendant. It publishes most of
the properties of TPanel except the bevel properties. It
borrows the BorderWidth property to set the shadow width and
implements 2 new properties, FShadowColor and FBackColor. The
use of these properties is self explanatory.

TShadowPanel has no real use beside having the raised and shadowed
look. It might make a nice pseudo shadowed window by dropping it
on a form and setting Align to client. It also looks nice for a
raised memo.

Anyway I hope you find this code useful. If you make improvements
please e-mail me a copy. Enjoy.

                Paul Warren
       HomeGrown Software Development
     (c) 1995 Langley British Columbia.
              (604) 530-9097
       e-mail:  hg_soft@uniserve.com
  Home page: http://haven.uniserve.com/~hg_soft

********************************************************}
unit Shdowpnl;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, Menus;

type
  TShadowPanel = class(TCustomPanel)
  private
    { Private declarations }
    FShadowColor: TColor;
    FBackColor: TColor;
  protected
    { Protected declarations }
    procedure Paint; override;
    procedure AlignControls(AControl: TControl; var Rect: TRect); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
  published
    { Published declarations }
    property Align;
    property Alignment;
    property BorderWidth;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Caption;
    property Color;
    property Ctl3D;
    property Font;
    property Locked;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property ShadowColor: TColor read FShadowColor write FShadowColor default clGray;
    property BackColor: TColor read FBackColor write FBackColor default clSilver;
  end;

procedure Register;

implementation

constructor TShadowPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [csAcceptsControls, csCaptureMouse, csClickEvents,
    csSetCaption, csOpaque, csDoubleClicks];
  Width := 185;
  Height := 41;
  Alignment := taCenter;
  BevelOuter := bvNone;
  BevelWidth := 1;
  BorderStyle := bsNone;
  BorderWidth := 4;
  Color := clWhite;
  FShadowColor := clGray;
  FBackColor := clSilver;
end;

procedure TShadowPanel.AlignControls(AControl: TControl; var Rect: TRect);
begin
  InflateRect(Rect, BorderWidth div 2, BorderWidth div 2);
  OffSetRect(Rect, -BorderWidth div 2, -BorderWidth div 2);
  inherited AlignControls(AControl, Rect);
end;

procedure TShadowPanel.Paint;
var
  Rect: TRect;
  Text: array[0..255] of Char;
  FontHeight: Integer;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
begin
  Rect := GetClientRect;
  { Draw a clear rect }
  Canvas.Brush.Style := bsClear;
  Canvas.Brush.Color := BackColor;
  Canvas.FillRect(Rect);
  { draw the shadow }
  InflateRect(Rect, -BorderWidth div 2, -BorderWidth div 2);
  OffSetRect(Rect, BorderWidth div 2, BorderWidth div 2);
  Canvas.Brush.Style := bsSolid;
  Canvas.Brush.Color := ShadowColor;
  Canvas.FillRect(Rect);
  OffSetRect(Rect, -BorderWidth, -BorderWidth);
  with Canvas do
  begin
    Brush.Color := Color;
    FillRect(Rect);
    Brush.Style := bsClear;
    Font := Self.Font;
    FontHeight := TextHeight('W');
    with Rect do
    begin
      Top := ((Bottom + Top) - FontHeight) shr 1;
      Bottom := Top + FontHeight;
    end;
    StrPCopy(Text, Caption);
    DrawText(Handle, Text, StrLen(Text), Rect, (DT_EXPANDTABS or
      DT_VCENTER) or Alignments[Alignment]);
  end;
end;

procedure Register;
begin
  RegisterComponents('Standard', [TShadowPanel]);
end;

end.
